@@ -107,20 +107,20 @@ class CloudyModel(object):
         """
 
         self.log_ = pc.log_
-        self.calling = 'CloudyModel'
         if verbose is not None:
             self.log_.level = verbose
         self.model_name = model_name
+        self.info = '<Cloudy model from {0}>'.format(self.model_name) 
+        self.calling = 'CloudyModel {0.model_name}'.format(self)
         self.log_.message('Creating CloudyModel for {0.model_name}'.format(self), calling = self.calling)
         self.model_name_s = model_name.split('/')[-1]
         self.line_is_log = line_is_log
         self.emis_is_log = emis_is_log
-        self.distance = distance  
+        self.distance = distance
+        self.empty_model = True
         self._read_stout()
-        if read_all_ext and self.out_exists:
-
+        if self.out_exists and not self.aborted and read_all_ext:
             self._init_all2zero()
-            
             if read_rad:
                 self._init_rad()
             if read_phy:
@@ -192,6 +192,7 @@ class CloudyModel(object):
         self.gdsize = None
         self.gdgrat_labels = None        
         self.plan_par = None
+        self.Hbeta_full = None
     
     def _init_rad(self):
         key = 'rad'
@@ -303,7 +304,10 @@ class CloudyModel(object):
             self.log_.message('Number of emissivities: {0.n_emis:d}'.format(self), calling = self.calling)
             self.emis_full = np.zeros((self.n_emis, np.size(emis)))
             for i, label in enumerate(self.emis_labels):
-                self.emis_full[i] = trans_emis(emis[label])                   
+                self.emis_full[i] = trans_emis(emis[label])
+            if 'H__1__4861A' in self.emis_labels:
+                self.Hbeta_full = (self.get_emis('H__1__4861A') * self.dvff).cumsum()
+                self.__Hbeta_cut = self.Hbeta_full[-1]
 
     def _init_ionic(self, elem, str_key = 'ele_'):
         key = str_key + elem
@@ -392,7 +396,6 @@ class CloudyModel(object):
     def _read_stout(self):
         self.out = {}
         file_name = self.model_name + '.out'
-        self.date_model = time.ctime(os.path.getctime(file_name))
         self.C3D_comments = []
         self.comments = []
         self.warnings = []
@@ -401,17 +404,20 @@ class CloudyModel(object):
             file_ = open(file_name, 'r')
             self.out_exists = True
         except:
-            self.log_.error(file_name + ' NOT read.', calling = self.calling)
+            self.log_.warn(file_name + ' NOT read.', calling = self.calling)
             self.out_exists = False
+            self.info = '<!!! Model {0} without output file>'.format(self.model_name)
             return None
-        
+        self.date_model = time.ctime(os.path.getctime(file_name))
         self.cloudy_version = file_.readline().strip()
         self.cloudy_version_major = pc.sextract(self.cloudy_version,'Cloudy ','.')
         self.Teff = None
+        self.out['Cloudy ends'] = ''
+        self.out['stop'] = ''
         for line in file_:
             if line[0:8] == ' ####  1':
                 self.out['###First'] = line
-            elif line[0:5] == ' ###':
+            elif line[0:4] == ' ###':
                 self.out['###Last'] = line
             elif 'Calculation stopped' in line:
                 self.out['stop'] = line
@@ -422,6 +428,8 @@ class CloudyModel(object):
                     self.out['Chem' + str(i + 1)] = file_.next()
             elif 'Grain Chemical Composition' in line:
                 self.out['GrainChem'] = file_.next()
+            elif 'Dust to gas ratio' in line:
+                self.out['D/G'] = line
             elif 'iterate' in line:
                 self.out['iterate'] = line
             elif 'Hi-Con' in line:
@@ -506,26 +514,37 @@ class CloudyModel(object):
         self.Phi0 = self.Phi.sum()
         
         self.abund = {}
-        Chem = self.out['Chem1'][0:-1]
-        if self.out['Chem2'] != ' \n':
-            Chem += self.out['Chem2'][0:-1]
-            if self.out['Chem3'] != ' \n':
-                Chem += self.out['Chem3'][0:-1]
-                if self.out['Chem4'] != ' \n':
-                    Chem += self.out['Chem4'][0:-1]
-        self.gas_mass_per_H = 0.
-        for ab_str in LIST_ALL_ELEM:
-            if len(ab_str) == 1:
-                sub1 = ab_str + ' : '
-            else:
-                sub1 = ab_str + ': '
-            try:             
-                self.abund[ab_str] = float(pc.sextract(Chem, sub1, 7))
-            except:
-                self.log_.message(ab_str + ' abundance not defined', calling = self.calling)
-            if (ab_str in ATOMIC_MASS) and (ab_str in self.abund):
-                self.gas_mass_per_H += 10**self.abund[ab_str] * ATOMIC_MASS[ab_str]
-                
+        try:
+            Chem = self.out['Chem1'][0:-1]
+            chem_is_ok = True
+            if self.out['Chem2'] != ' \n':
+                Chem += self.out['Chem2'][0:-1]
+                if self.out['Chem3'] != ' \n':
+                    Chem += self.out['Chem3'][0:-1]
+                    if self.out['Chem4'] != ' \n':
+                        Chem += self.out['Chem4'][0:-1]
+            self.gas_mass_per_H = 0.
+            for ab_str in LIST_ALL_ELEM:
+                if len(ab_str) == 1:
+                    sub1 = ab_str + ' : '
+                else:
+                    sub1 = ab_str + ': '
+                try:             
+                    self.abund[ab_str] = float(pc.sextract(Chem, sub1, 7))
+                except:
+                    self.log_.message(ab_str + ' abundance not defined', calling = self.calling)
+                if (ab_str in ATOMIC_MASS) and (ab_str in self.abund):
+                    self.gas_mass_per_H += 10**self.abund[ab_str] * ATOMIC_MASS[ab_str]
+        except:
+            chem_is_ok = False
+        if ("ABORT" in self.out['Cloudy ends']) or ("aborted" in self.out['stop']) or (not chem_is_ok):
+            self.aborted = True
+            self.log_.warn('Model aborted', calling = self.calling)
+            self.info = '<!!! Model {0} aborted>'.format(self.model_name)
+        else:
+            self.aborted = False
+        return True
+    
     def read_outputs(self, extension, delimiter='\t', comments=';', names=True, **kwargs):
         file_ = self.model_name + '.' + extension
         if os.path.exists(file_):
@@ -1160,8 +1179,8 @@ class CloudyModel(object):
         elif 'Cont__nu' in self._res['cont'].dtype.names:
             hnu = self._res['cont']['Cont__nu']
         else:
-            self.log_.error('Hnu NOT defined in the continuum', calling = self.calling)
-            hnu = None
+            self.log_.warn('Hnu NOT defined in the continuum', calling = self.calling)
+            return None
         if unit == 'Ryd':
             to_return = hnu
         elif unit == 'eV':
@@ -1349,6 +1368,18 @@ class CloudyModel(object):
             
     H_mass_cut = property(_get_H_mass_cut, _set_H_mass_cut, None, None)
 
+    def _get_Hbeta_cut(self):
+        return self.__Hbeta_cut
+    
+    def _set_Hbeta_cut(self, value):
+        if value > self.Hbeta_full[1]:
+            self.r_out_cut = self.radius_full[self.Hbeta_full <= value][-1]
+            self.__Hbeta_cut = self.Hbeta
+        else:
+            self.log_.warn('Hbeta_cut must be greater than minimal value', calling = self.calling)
+
+    Hbeta_cut = property(_get_Hbeta_cut, _set_Hbeta_cut, None, None)
+
     ## Hp_mass = \f$ \int m_H.n_{H^+}.ff.dV\f$ [solar mass]
     @property        
     def Hp_mass(self):
@@ -1379,6 +1410,17 @@ class CloudyModel(object):
             self.log_.warn('H mass_tot not available', calling = self.calling)
             return None
         
+    ## Hbeta = \f$ \int Hbeta.n_H.ff.dV\f$ [solar mass]                
+    @property        
+    def Hbeta(self):
+        """Return the intensity of Hbeta"""
+        try:
+            return self.get_emis_vol('H__1__4861A')
+        except:
+            self.log_.warn('H beta not available', calling = self.calling)
+            return None
+        
+    
     ## Mean Temperature \f$T0=\frac{\int T_e.n_e.n_H.ff.dV}{\int n_e.n_H.ff.dV}\f$
     @property        
     def T0(self):
@@ -1407,7 +1449,43 @@ class CloudyModel(object):
             return self.get_emis_vol('H__1__4861A') / (self.r_out_cut**2 * np.pi * 206265.**2)
         else:
             self.log_.warn('Hbeta emissivity not in emis file', calling = self.calling + '.get_Hb_SB')
-        
+    
+    ## Hb_EW = -$\lambda_\beta$ x I$_\beta^{line}$ / $\lambda.F_\beta^{cont}$
+    def get_Hb_EW(self):
+        """
+        Hbeta Equivalent Width:
+        Returns -4861 * I(H__1__4861A) / continuum(4860)
+        where continuum(4860) is estimated by looking for the minimum of the net transmited continuum between
+            4560 and 4860 on one side, and 4860 and 5160 on the other side, and meaning them.
+        """
+        if 'H__1__4861A' in self.emis_labels:
+            mask_low = (self.get_cont_x('Ang') < 4860) & (self.get_cont_x('Ang') > 4560)
+            I_low = np.min(self.get_cont_y('ntrans', 'es')[mask_low])
+            mask_high = (self.get_cont_x('Ang') > 4860) & (self.get_cont_x('Ang') < 5160)
+            I_high = np.min(self.get_cont_y('ntrans', 'es')[mask_high])
+            return -4861 * self.get_emis_vol('H__1__4861A') / np.mean((I_low, I_high)) 
+        else:
+            self.log_.warn('Hbeta line not in emis file', calling = self.calling + '.get_Hb_EW')
+            return None
+
+    ## Ha_EW = -$\lambda_\alpha$ x I$_\alpha^{line}$ / $\lambda.F_\alpha^{cont}$
+    def get_Ha_EW(self):
+        """
+        Hbeta Equivalent Width:
+        Returns -6563 * I(H__1__6563A) / continuum(6563)
+        where continuum(6563) is estimated by looking for the minimum of the net transmited continuum between
+            6260 and 6560 on one side, and 6560 and 6860 on the other side, and meaning them.
+        """
+        if 'H__1__6563A' in self.emis_labels:
+            mask_low = (self.get_cont_x('Ang') < 6560) & (self.get_cont_x('Ang') > 6260)
+            I_low = np.min(self.get_cont_y('ntrans', 'es')[mask_low])
+            mask_high = (self.get_cont_x('Ang') > 6560) & (self.get_cont_x('Ang') < 6860)
+            I_high = np.min(self.get_cont_y('ntrans', 'es')[mask_high])
+            return -6563 * self.get_emis_vol('H__1__6563A') / np.mean((I_low, I_high)) 
+        else:
+            self.log_.warn('Halpha line not in emis file', calling = self.calling + '.get_Hb_EW')
+            return None
+
     ## is_valid_ion(elem, ion) return True if elem, ion is available in get_ionic.
     def is_valid_ion(self, elem, ion):        
         """
@@ -1536,10 +1614,10 @@ class CloudyModel(object):
             pass
         
     def __repr__(self):
-        return "<Cloudy model from {0.model_name}>".format(self)
+        return "{0.info}".format(self)
     
     def __str__(self):
-        return "<Cloudy model from {0.model_name}>".format(self)
+        return "{0.info}".format(self)
     
 ## @include copyright.txt
 def load_models(model_name = None, mod_list = None, n_sample = None, verbose = False, **kwargs):
@@ -1571,7 +1649,9 @@ def load_models(model_name = None, mod_list = None, n_sample = None, verbose = F
             model_name = outfile[0:-4]
         else:
             model_name = outfile
-        m.append(CloudyModel(model_name, **kwargs))
+        cm = CloudyModel(model_name, **kwargs)
+        if not cm.aborted:
+            m.append(cm)
         if verbose:
             print  '{0} model read'.format(outfile[0:-4])
     pc.log_.message('{0} models read'.format(np.size(mod_list)), calling = 'load_models')
@@ -1614,6 +1694,7 @@ class CloudyInput(object):
         self._save_list_elems = pc.config.SAVE_LIST_ELEMS
         self.save_list = pc.config.SAVE_LIST
         self._save_list_grains = pc.config.SAVE_LIST_GRAINS
+        self.cloudy_version = None
         
     def set_save_str(self, save = 'save'):
         """
@@ -1735,13 +1816,15 @@ class CloudyInput(object):
             if 'sphere' in self._input:
                 del self._input['sphere']
             
-    def set_iterate(self, n_iter = None):
+    def set_iterate(self, n_iter = None, to_convergence = False):
         """
         Set the iterate parameter.
         Parameter:
             - n_iter: If None, set the iterate parameter to "iterate" in the Cloudy input file,
                 if ==0, unset the iterate (nothing will be printed), otherwise set iterate to the
                 value of n_iter.
+            - to_convergence [False]: If True, iterate to convergence is printed out. 
+                n_iter without effect then
         """
         if n_iter is None:
             self._input['iterate'] = 'iterate'
@@ -1750,6 +1833,8 @@ class CloudyInput(object):
                 del self._input['iterate']
         else:
             self._input['iterate'] = 'iterate {0:d}'.format(n_iter)
+        if to_convergence:
+            self._input['iterate'] = 'iterate to convergence'
 
     def set_grains(self, grains = None):
         """
@@ -2079,7 +2164,8 @@ class CloudyInput(object):
         """
         if model_name is None:
             model_name = self.model_name
-        run_cloudy(dir_ = dir_, n_proc = n_proc, use_make = use_make, model_name = model_name, precom=precom)
+        run_cloudy(dir_ = dir_, n_proc = n_proc, use_make = use_make, model_name = model_name, precom=precom,
+                   cloudy_version=self.cloudy_version)
     
     def print_make_file(self, dir_ = None):
         """
@@ -2118,7 +2204,7 @@ all: $(OBJ)
     makefile.close()
 
 ## Function used to run Cloudy on input files.                
-def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None, precom=""):
+def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None, precom="", cloudy_version=None):
     """
     Run a (set of ) cloudy model(s)
     
@@ -2130,9 +2216,15 @@ def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None, prec
         - model_name:  if not None, used by: make name="model_name" or cloudy < model_name.in
             if None and use_make, make will run any pending model
         - precom: a string to put before Cloudy (e.g. "\nice 10")
+        - cloudy_version: one of the keys of pc.config.cloudy_dict, pointing to the location of the executable,
+            e.g. '10.00' or '13.03'. If set to None (default), then pc.config.cloudy.exe is used
     """
     if dir_ is None:
         dir_ = '/'.join(model_name.split('/')[0:-1])
+    cloudy_exe = pc.config.cloudy_exe
+    if cloudy_version is not None:
+        if cloudy_version in pc.config.cloudy_dict:
+            cloudy_exe = pc.config.cloudy_dict[cloudy_version]
     if use_make:
         to_run = 'cd {0} ; make -j {1:d}'.format(dir_, n_proc)
         if model_name is not None:
@@ -2143,7 +2235,7 @@ def run_cloudy(dir_ = None, n_proc = 1, use_make = True, model_name = None, prec
         if model_name is None:
             pc.log_.error('Model name must be set', calling = 'run_cloudy')
         else:
-            to_run = 'cd {0} ; {1} {2}'.format(dir_, precom, pc.config.cloudy_exe)
+            to_run = 'cd {0} ; {1} {2}'.format(dir_, precom, cloudy_exe)
             stdin = file('{0}/{1}.in'.format(dir_, model_name.split('/')[-1]), 'r')
             stdout = file('{0}/{1}.out'.format(dir_, model_name.split('/')[-1]), 'w')   
     pc.log_.message('running: {0}'.format(to_run), calling = 'run_cloudy')
