@@ -1,12 +1,13 @@
 import os
+import subprocess
 import numpy as np
 import pyCloudy as pc
 from pyCloudy.utils.init import LIST_ELEM
 from pyCloudy.utils.logging import my_logging
 if pc.config.INSTALLED['MySQL']:
     import MySQLdb
-
-import subprocess
+if pc.config.INSTALLED['pandas']:
+     import pandas.io.sql as psql
 
 def _sql2numpy(sqltype):
     if sqltype == 'float':
@@ -46,12 +47,12 @@ class MdB(object):
         
         Latter, we will also use the ODBC connector. Install the connector from MySQl: http://dev.mysql.com/downloads/connector/odbc/
         and then use pyodbc with:
-        cnxn = pyodbc.connect('DRIVER={MySQL ODBC 5.2 Driver};SERVER=132.248.1.102;DATABASE=OVN2;UID=OVN_admin2;PWD=oiii4363;SOCKET=/var/mysql/mysql.sock')
+        cnxn = pyodbc.connect('DRIVER={MySQL ODBC 5.2 Driver};SERVER=127.0.0.1;DATABASE=OVN;UID=OVN_user;PWD=oiii5007;SOCKET=/var/mysql/mysql.sock')
         
         """
 
 
-        self.log_ = MdB.MdBlog_
+        self.log_ = self.__class__.MdBlog_
         self.calling = 'MdB'
         if OVN_dic is not None:
             if 'base_name' in OVN_dic:
@@ -140,12 +141,18 @@ class MdB(object):
             self.log_.warn('Not connected', calling = self.calling)
 
     def exec_dB(self, command, format_ = 'dict'):
-        if format_ not in ('dict', 'tuple', 'numpy', 'dict2'):
+        if format_ not in ('dict', 'tuple', 'numpy', 'dict2', 'pandas'):
             self.log_.error('format"{0}" not recognized'.format(format_), calling = self.calling)
         if not self.connected:
             self.log_.error('Not connected to a database', calling = self.calling)
             return None
         self.log_.message('Command sent: {0}'.format(command), calling = self.calling)
+        if format_ == 'pandas':
+            if pc.config.INSTALLED['pandas']:
+                res = psql.frame_query(command, con=self._dB)
+                return res, len(res)
+            else:
+                pc.log_.error('pandas is not available, use another format', calling=self.calling)
         if format_[0:4] == 'dict':
             cursor = self._cursor
         else:
@@ -160,8 +167,8 @@ class MdB(object):
             self.log_.error('Error on reading result of {0}'.format(command), calling = self.calling)
         return res, N
             
-    def select_dB(self, select_ = '*', from_ = 'OVN.tab1', where_ = None, order_ = None, group_ = None, limit_ = 1,
-                  format_ = 'dict', dtype_ = None):
+    def select_dB(self, select_ = '*', from_ = 'OVN.tab1', where_ = None, order_ = None, group_ = None, 
+                  limit_ = 1, format_ = 'dict', dtype_ = None):
         """
         Usage:
             dd, n = mdb.select_dB(select_ = 'L_1, L_26, L_21', 
@@ -278,17 +285,6 @@ class MdB(object):
                     dtype_list.append((name, ntype))        
         return np.dtype(dtype_list)
             
-    def print_all_refs(self, from_ = 'OVN.tab1'):
-        res, N_ref = self.select_dB(select_ = 'distinct(ref), count(*)', from_ = from_, group_ = 'ref', limit_ = None)
-        for row in res:
-            print('The ref "{0:15}" counts {1:8d} entries.'.format(row['ref'], row['count(*)']))
-        print('Number of distinct references = {0}'.format(N_ref))
-        
-    def print_all_lines(self, limit_ = None):
-        lines, N_lines = self.select_dB(select_ = '*', from_ = 'OVN.lines', limit_ = limit_)
-        for line in lines:
-            print('N = {0[N]:3} id = {0[id]:4} label = {0[label]:5} wavelength = {0[lambda]:6} full name = {0[name]}'.format(line))
-
     def __repr__(self):
         if self.connected:
             return "<MdB connected to {0.base_name}@{0.host}>".format(self)
@@ -299,12 +295,13 @@ class MdB_subproc(object):
     """
     Alternative way, when MySQLdb not available. Still in development.
     """
+    MdBlog_ = my_logging()
     def __init__(self, OVN_dic = None, base_name = 'OVN',tmp_base_name = 'OVN_tmp', 
                  user_name = 'OVN_user', user_passwd = 'getenv', 
                  host = 'localhost', unix_socket = '/var/mysql/mysql.sock', port = 3306,
                  connect = True):
         
-        self.log_ = MdB.MdBlog_
+        self.log_ = self.__class__.MdBlog_
         self.calling = 'MdB'
         if OVN_dic is not None:
             if 'base_name' in OVN_dic:
@@ -339,6 +336,9 @@ class MdB_subproc(object):
     def connect_dB(self):
         pass
     
+    def close_dB(self):
+        pass
+    
     def exec_dB(self, command, format_ = 'dict'):
         if format_ not in ('dict', 'tuple', 'numpy', 'dict2'):
             self.log_.error('format"{0}" not recognized'.format(format_), calling = self.calling)
@@ -351,7 +351,8 @@ class MdB_subproc(object):
         proc = subprocess.Popen(["mysql", 
                                  "--host={0}".format(self.host),
                                  "--user={0}".format(self.user_name), 
-                                 "--password={0}".format(self.user_passwd), 
+                                 "--password={0}".format(self.user_passwd),
+                                 "--port={0}".format(self.port),
                                  "{0}".format(self.base_name)],
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE)
@@ -367,7 +368,7 @@ class MdB_subproc(object):
         return out, N
 
     def select_dB(self, select_ = '*', from_ = 'OVN.tab1', where_ = None, order_ = None, group_ = None, limit_ = 1,
-                  format_ = 'dict', dtype_ = None):
+                  format_ = 'dict2', dtype_ = None):
         """
         Usage:
             dd, n = mdb.select_dB(select_ = 'L_1, L_26, L_21', 
@@ -385,12 +386,24 @@ class MdB_subproc(object):
             req += 'GROUP BY {0} '.format(group_)
         if limit_ is not None:
             req += 'LIMIT {0:d}'.format(limit_)
-        res, N = self.exec_dB(req, format_ = format_)
+        res_tmp, N = self.exec_dB(req, format_ = format_)
         try:
-            for i,line in enumerate(res):
-                res[i] = line.split('\t')
+            for i,line in enumerate(res_tmp):
+                res_tmp[i] = line.split('\t')
         except:
             pass
-        
-    
+        if N == 0:
+            res = None
+            return res,N
+        if format_ in ('dict', 'dict2'):
+            res = {}
+            resnp = np.array(res_tmp[1:-1])
+            for i, key in enumerate(res_tmp[0]):
+                try:
+                    res[key] = np.array(resnp[:,i], dtype='float')
+                except:
+                    res[key] = resnp[:,i]
+        return res,N-2
+            
+            
     
\ No newline at end of file
