@@ -14,6 +14,9 @@ class _Config(object):
             self.cloudy_exe = os.environ['CLOUDY_EXE']
         else:
             self.cloudy_exe = 'cloudy.exe'
+        
+        self.cloudy_dict = {'10.00': '/usr/local/Cloudy/c10.00/cloudy.exe',
+                       '13.03': '/usr/local/Cloudy/c13.03/cloudy.exe'}
             
         self.INSTALLED ={}
         try:
@@ -51,6 +54,11 @@ class _Config(object):
             self.INSTALLED['MySQL'] = True
         except:
             self.INSTALLED['MySQL'] = False
+        try:
+            import pandas
+            self.INSTALLED['pandas'] = True
+        except:
+            self.INSTALLED['pandas'] = False
             
         try:
             test_str = ['#one\t two\t three', '1\t 2\t 3']
